/*****************************************************************************/
/* $Header: /cvsroot/umbrella/umbrella-devel/kernel/security/umbrella/umb_bitvector.c,v 1.17 2005/03/11 09:48:26 ipqw Exp $ */
/*****************************************************************************/
/*  AUTHOR:                                                                  */
/*****************************************************************************/
/*                                                                           */
/*    Sren Nhr Christensen                                                 */
/*    Kristian Srensen                                                      */
/*    Michel Thryse                                                         */
/*    mailto:umbrella@cs.aau.dk                                              */
/*    http://umbrella.sourceforge.net                                        */
/*                                                                           */
/*****************************************************************************/
/*  COPYRIGHT:                                                               */
/*****************************************************************************/
/*                                                                           */
/*    Copyright (c) 2004 The Umbrella Security Team.                         */
/*    All rights reserved.                                                   */
/*                                                                           */
/*****************************************************************************/
/*  LICENSE:                                                                 */
/*****************************************************************************/
/*                                                                           */
/*    This file is part of the Umbrella Security Project and is free         */
/*    software; you can redistribute it and/or modify it under the terms     */
/*    of the GNU Library General Public License as published by the Free     */
/*    Software Foundation;                                                   */
/*    The code is distributed in the hope that it will be useful,            */
/*    but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       */
/*    Library General Public License for more details.                       */
/*                                                                           */
/*    You should have received a copy of the GNU Library General Public      */
/*    License along with this library; if not, write to the                  */
/*    Free Software Foundation, Inc.,                                        */
/*    59 Temple Place, Suite 330, Boston, MA 02111-1307 USA                  */
/*                                                                           */
/*    or download a copy from ftp://ftp.gnu.org/pub/gnu/COPYING.LIB-2.0      */
/*                                                                           */
/*****************************************************************************/
#include <linux/types.h>
#include <linux/config.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/init.h>

#include "include/umb_bitvector.h"



bitvector bv_create(void) {

	bitvector vector = (bitvector) kmalloc(32, GFP_ATOMIC);
	if (!vector)
		printk("Umbrella: umb_bitvector.c: memory allocation failed\n");

	*vector = 0; /* initialize to 32 times 0 */
	return vector;

}



void bv_destroy(bitvector vector) {

	kfree(vector);

}



void bv_bit_on(bitvector vector, int index) {

	int org_vector = *vector;
	*vector >>= index;
	*vector = *vector | 1;
	*vector <<= index;
	*vector = org_vector | *vector;

}



void bv_or(bitvector result, bitvector a, bitvector b) {

	*result = *a | *b;

}


void bv_copy(bitvector result, bitvector a) {

	*result = *a;

}



void bv_reset(bitvector vector) {
	
	*vector = 0;

}



int bv_testbit(const bitvector vector, int index) {

	int tmp = *vector;

	tmp >>= index;
	return tmp & 1;

}



void bv_print(bitvector vector) {

	int i;

	printk("Umbrella: bitvector: ");
	for (i = 0; i < 32; i++) printk("%i", bv_testbit(vector, i));
	printk("\n");

}
