/*****************************************************************************/
/* $Header: /cvsroot/umbrella/umbrella-devel/kernel/security/umbrella/umb_keyring.c,v 1.8 2005/04/12 08:07:07 sonoch Exp $ */
/*****************************************************************************/
/*  AUTHOR:                                                                  */
/*****************************************************************************/
/*                                                                           */
/*    Sren Nhr Christensen                                                 */
/*    Kristian Srensen                                                      */
/*    Michel Thryse                                                         */
/*    mailto:umbrella@cs.aau.dk                                              */
/*    http://umbrella.sourceforge.net                                        */
/*                                                                           */
/*****************************************************************************/
/*  COPYRIGHT:                                                               */
/*****************************************************************************/
/*                                                                           */
/*    Copyright (c) 2004 The Umbrella Security Team.                         */
/*    All rights reserved.                                                   */
/*                                                                           */
/*****************************************************************************/
/*  LICENSE:                                                                 */
/*****************************************************************************/
/*                                                                           */
/*    This file is part of the Umbrella Security Project and is free         */
/*    software; you can redistribute it and/or modify it under the terms     */
/*    of the GNU Library General Public License as published by the Free     */
/*    Software Foundation;                                                   */
/*    The code is distributed in the hope that it will be useful,            */
/*    but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       */
/*    Library General Public License for more details.                       */
/*                                                                           */
/*    You should have received a copy of the GNU Library General Public      */
/*    License along with this library; if not, write to the                  */
/*    Free Software Foundation, Inc.,                                        */
/*    59 Temple Place, Suite 330, Boston, MA 02111-1307 USA                  */
/*                                                                           */
/*    or download a copy from ftp://ftp.gnu.org/pub/gnu/COPYING.LIB-2.0      */
/*                                                                           */
/*****************************************************************************/
#include "include/umb_signature.h"
#include "include/umb_keyring.h"
#include <asm/uaccess.h>

struct key *global_keyring[] = {NULL, NULL, NULL, NULL};

static struct key *insert_masterkey(char *file) {

	MPI MPI_e, MPI_n  = NULL;
	struct key *key = (struct key *)kmalloc(sizeof(struct key), GFP_ATOMIC);

	int i = 3; /* value?? ask MT */
	int j = 130; /* value?? ask MT */

	unsigned char *n =
		(unsigned char *)kmalloc(UMBRELLA_MPI_MAX_SIZE_N, GFP_ATOMIC);
	unsigned char *e =
		(unsigned char *)kmalloc(UMBRELLA_MPI_MAX_SIZE_E, GFP_ATOMIC);

	key->id = (char *)kmalloc(UMB_KEY_ID_SIZE, GFP_ATOMIC);
	
	if(!n || !e || !key->id) {
		kfree(n);
		kfree(e);
		kfree(key->id);
		return NULL;
	}

	if (umb_get_pkey(n, e, file) == -1)
		return NULL;
	
	MPI_e = mpi_read_from_buffer(e, &i, 0);
	MPI_n = mpi_read_from_buffer(n, &j, 0);

	key->payload[0] = MPI_n;
	key->payload[1] = MPI_e;
	strcpy(key->id, "master\0");
	
	kfree(n);
	kfree(e);
	
	return key;
}

int init_keyring() {

	struct key *key = NULL;

	printk(KERN_INFO "Initializing keyring!\n");
	key = insert_masterkey("/etc/umbrella/keyring/master.pub");

	if(!key) 
		return -1;
	
	global_keyring[0] = key;
	
	return 0;
}

struct key *get_key(char *key_id) {

	int position = 0;

	/* init keyring, if no masterkey is found! */
	if (!global_keyring[0]) {
		printk(KERN_INFO "No key at pos. 0, trying to initialize keyring\n");
		if (init_keyring() == -1) {
			printk(KERN_INFO "Keyring init. failed\n");
			return NULL;
		}
	}

	/* find the key! */
	while (global_keyring[position] && position < KEYRING_LENGTH) {
		if(!strcmp(global_keyring[position]->id, key_id))
			return global_keyring[position];
		position++;
	}
	printk(KERN_INFO "Umbrella: No matching key was found in keyring!\n");
	return NULL;
}
                                                                                
void umb_key_insert(char *rawkey, char *keyid) {

	MPI MPI_e, MPI_n  = NULL;
	struct key *key = (struct key *)kmalloc(sizeof(struct key), GFP_ATOMIC);

	int i = 3; /* value?? ask MT */
	int j = 130; /* value?? ask MT */
	
	unsigned char *n =
		(unsigned char *)kmalloc(UMBRELLA_MPI_MAX_SIZE_N, GFP_ATOMIC);
	unsigned char *e =
		(unsigned char *)kmalloc(UMBRELLA_MPI_MAX_SIZE_E, GFP_ATOMIC);

	key->id = (char *)kmalloc(UMB_KEY_ID_SIZE, GFP_ATOMIC);

	if(!n || !e || !key || !key->id) {
		kfree(n);
		kfree(e);
		kfree(key);
		kfree(key->id);
		printk("Umbrella: insert key ENOMEM!\n");
		return;
	}
		
	if (umb_get_pkey_from_mem(n, e, rawkey) == -1) {
		printk("Umbrella: insert key could not read key from memory!\n");
		return;
	}
	MPI_e = mpi_read_from_buffer(e, &i, 0);
	MPI_n = mpi_read_from_buffer(n, &j, 0);
	kfree(n);
	kfree(e);
	
	key->payload[0] = MPI_n;
	key->payload[1] = MPI_e;
	strcpy(key->id, keyid);  

	/* find a free space in keyring */
	i = 0;
	while(global_keyring[i] && i<4)
		i++;
	
	if(global_keyring[i] != NULL) {
		printk(KERN_INFO "Umbrella: No space for new key in keyring!\n");
		kfree(key);
		kfree(key->id);
		return;
	}
		
	global_keyring[i] = key;
	
	return;
}
