/******************************************************************************/
/* $Header: /cvsroot/umbrella/umbrella-devel/kernel/security/umbrella/umb_types.c,v 1.26 2005/04/13 12:19:39 ipqw Exp $ */
/******************************************************************************/
/*  AUTHOR:                                                                   */
/******************************************************************************/
/*                                                                            */
/*    Sren Nhr Christensen                                                  */
/*    Kristian Srensen                                                       */
/*    Michel Thryse                                                          */
/*    mailto:umbrella@cs.auc.dk                                               */
/*    http://umbrella.sourceforge.net                                         */
/*                                                                            */
/******************************************************************************/
/*  COPYRIGHT:                                                                */
/******************************************************************************/
/*                                                                            */
/*    Copyright (c) 2004 The Umbrella Security Team.                          */
/*    All rights reserved.                                                    */
/*                                                                            */
/******************************************************************************/
/*  LICENSE:                                                                  */
/******************************************************************************/
/*                                                                            */
/*    This file is part of the Umbrella Security Project and is free          */
/*    software; you can redistribute it and/or modify it under the terms      */
/*    of the GNU Library General Public License as published by the Free      */
/*    Software Foundation;                                                    */
/*    The code is distributed in the hope that it will be useful,             */
/*    but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        */
/*    Library General Public License for more details.                        */
/*                                                                            */
/*    You should have received a copy of the GNU Library General Public       */
/*    License along with this library; if not, write to the                   */
/*    Free Software Foundation, Inc.,                                         */
/*    59 Temple Place, Suite 330, Boston, MA 02111-1307 USA                   */
/*                                                                            */
/*    or download a copy from ftp://ftp.gnu.org/pub/gnu/COPYING.LIB-2.0       */
/*                                                                            */
/******************************************************************************/

#include <linux/security.h>
#include "include/umb_types.h"
#include "include/umb_bitvector.h"
#include "include/umb_signature.h"
#include "include/umb_fsr.h"

/******************************************************************************/
/* UMBRELLA TYPES: security struct memory management functions                */
/******************************************************************************/

struct security_struct *security_struct_create() {
	struct security_struct* security;
	int i = 0;

	security = (struct security_struct *) kmalloc(sizeof(struct security_struct), GFP_ATOMIC);

	if (!security)
		return NULL;

	security->level1 = bv_create();
	security->child_level1 = bv_create();
	security->level2 = fsr_init();

	for (i = 0; i<MAX_L2; i++)
		security->child_level2[i] = NULL;


	/* extra fields in security struct for holding binary restrictions */
	security->bin_level1 = bv_create();

	for (i = 0; i<MAX_L2; i++)
		security->bin_level2[i] = NULL;

	if (!security->level1 || !security->child_level1 || !security->level2 || 
		!security->bin_level1 || !security->bin_level2 ) {
		printk("Umbrella: security_struct_create: Not enough memory!\n");
		return NULL;	
	}	
	return security;
}

void security_struct_destroy(struct security_struct *target){
	int i = 0;

	while (target->child_level2[i] != NULL)
		kfree(target->child_level2[i++]);

	while (target->bin_level2[i] != NULL)
		kfree(target->bin_level2[i++]);
	
	bv_destroy(target->bin_level1);
	bv_destroy(target->level1);
	bv_destroy(target->child_level1);
 	fsr_free(target->level2);
}


